"""Python client for ETI/Domo."""


DEBUG_DEEP = False

# Base library constants
VERSION = "0.6.1-alpha"
ISSUE_URL = "https://github.com/lrzdeveloper/python-came-manager/issues"

STARTUP_MESSAGE = f"""
-------------------------------------------------------------------
CAME ETI/Domo API Python client
Version: {VERSION}
If you have ANY issues with this you need to open an issue here:
{ISSUE_URL}
-------------------------------------------------------------------
"""
